<?php
// $Id: uscongress_thomas_screenscrape.inc,v 1.1.2.18 2008/02/22 19:17:20 douggreen Exp $

function uscongress_thomas_screenscrape_import($congress, $time = 10) {
  // return success if any bills are retrieved
  $success = 0;

  // keep track of the start time so that we can stop after the user-requested time interval
  $start = microtime(TRUE);

  // loop through the different bill types, getting the summary page for each
  $billtypes = drupal_map_assoc(_uscongress_thomas_billtypes());
  while (count($billtypes) > 0) {
    // decide which billtype to get next
    // this try's to split the time fairly amongst all the billtypes,
    // and skips ones that we know don't have any to check
    $billtype = _uscongress_thomas_get_next_billtype();
    if (!isset($billtypes[$billtype])) {
      continue;
    }

    // decide which bill to get next
    // first, look in the import log for bills that failed during the download, and retry them once a day
    if ($ref = db_result(db_query("SELECT ref FROM {uscongress_import} WHERE import = 'screenscrape' AND status = 0 AND ref LIKE '%s%%' AND timestamp + 86400 < %d ORDER BY timestamp ASC LIMIT 1", $billtype, time()))) {
      $billno = substr($ref, 2);
    }
    else {
      $billno = variable_get('uscongress_last_'. $billtype, 0);

      // don't retry a bill that recently errored out
      if (db_result(db_query("SELECT ref FROM {uscongress_import} WHERE import = 'screenscrape' AND status = 0 AND ref LIKE '%s%d'", $billtype, $billno))) {
        unset($billtypes[$billtype]);
        continue;
      }
    }

    // retrieve the summary page for the billtype
    $url = 'http://thomas.loc.gov/cgi-bin/bdquery/d?d'. $congress .':'. $billno .':./list/bss/d'. $congress . $billtype .'.lst:[[o]]';
    _uscongress_watchdog('screenscrape', $congress, $billtype . $billno, $url, t('GET List'), WATCHDOG_INFO);
    $result = drupal_http_request($url);
    if ($result->redirect_code == 200 || $result->code == 200) {
      // parse the summary page (and get additional pages as needed), stopping when the timer expires
      switch (_uscongress_thomas_parse_listing($result->data, $congress, $billtype, $time, $start)) {
        default: // error
          _uscongress_watchdog('screenscrape', $congress, $billtype . $billno, $url, t('Parse Error'));
          return 0;
        case -1: // processed at least one successfully, but timer expired
          return 1;
        case 1:
          $success = 1;
          break;
      }
    }
    else {
      // remember that this billtype returned an error
      _uscongress_watchdog('screenscrape', $congress, $billtype . $billno, $url, t('HTTP Error %code', array('%code' => $result->code)));
    }
  }
  return $success;
}

/**
 * get the page from Thomas again and refresh the information in our database
 */
function uscongress_thomas_screenscrape_update($congress, $billtype, $billno) {
  // save the basic information
  $info = array(
    'congressno' => $congress,
    'billtype' => $billtype,
    'source' => 'http://thomas.loc.gov/cgi-bin/bdquery/D?d'. $congress .':'. $billno .':./list/bss/d'. $congress . $billtype .'.lst:',
    'number' => $billno,
  );

  // retrieve the "All" page from Thomas and parse it
  $url = $info['source'] .'@@@L&summ2=m&';
  _uscongress_watchdog('screenscrape', $congress, $billtype . $billno, $url, t('GET Bill'), WATCHDOG_INFO);
  $result = drupal_http_request($url);
  if ($result->redirect_code == 200 || $result->code == 200) {
    $info['all'] = _uscongress_thomas_parse_all(str_replace("\n", " ", $result->data), $billtype);
  }
  else {
    _uscongress_watchdog('screenscrape', $congress, $billtype . $billno, $url, t('HTTP Error %code', array('%code' => $result->code)));
    $error = TRUE;
  }

  // retrieve the "Subject" page from Thomas and parse it
  $url = $info['source'] .'@@@J';
  $result = drupal_http_request($url);
  if ($result->redirect_code == 200 || $result->code == 200) {
    $info['subject'] = _uscongress_thomas_parse_subject(str_replace("\n", " ", $result->data));
  }
  else {
    _uscongress_watchdog('screenscrape', $congress, $billtype . $billno, $url, t('HTTP Error %code', array('%code' => $result->code)));
    $error = TRUE;
  }

  // save the bill
  if (_uscongress_thomas_add_bill_node($info) && !isset($error)) {
    _uscongress_set_import_status('screenscrape', $congress, $billtype . $billno, 1);
  }
}

function _uscongress_thomas_billtypes() {
  static $billtypes;
  if (!isset($billtypes)) {
    $billtype_settings = variable_get('uscongress_billtypes', 'HR');
    if (is_array($billtype_settings)) {
      $billtypes = array();
      foreach ($billtype_settings as $type => $enabled) {
        if ($enabled) {
          $billtypes[] = $type;
        }
      }
    }
    else {
      $billtypes = explode(' ', $billtype_settings);
    }
  }
  return $billtypes;
}

function _uscongress_thomas_get_next_billtype() {
  // initialize the next billindex
  static $index;
  if (!isset($index)) {
    $index = variable_get('uscongress_next_billindex', 0);
  }

  // return one of these types
  $billtypes = _uscongress_thomas_billtypes();

  // this is the current type
  $billtype = $billtypes[$index];

  // this is the next type
  $index = ($index + 1) % count($billtypes);
  variable_set('uscongress_next_billindex', $index);

  // return the current type
  return $billtype;
}

function _uscongress_thomas_parse_listing($html, $congress, $billtype, $time, $start) {
  // split the page into sections based on the query linking to the bill
  $regex = ',<a href="(/cgi-bin/bdquery/D\?d\d+:(\d+):\./list/bss/d\d+[SH][A-Z]\.\w+::)">[SH]\.[^0-9]*\d+\s*</A>:\s*,i';
  if ($bills = preg_split($regex, $html, -1, PREG_SPLIT_DELIM_CAPTURE)) {
    // skip the stuff at the top
    array_shift($bills);

    // loop through all of the bills and parse the summaries one at a time
    while (count($bills)) {
      // save the basic information
      // NOTE: the preg_split above breaks the html into an triplets
      // @TODO: get the session from somewhere
      $bill = array_splice($bills, 0, 3);
      $billno = $bill[1];
      $info = array(
        'congressno' => $congress,
        'billtype' => $billtype,
        'source' => 'http://thomas.loc.gov'. $bill[0],
        'number' => $billno,
      );

      // strip the next bill number from this string, which is getting appended because the above regex isn't perfect
      $billhtml = preg_replace('/(^:\s*|<hr>.*$)/i', '', str_replace("\n", " ", $bill[2]));

      // save the summary information
      $info['listing'] = _uscongress_thomas_parse_listing_summary('<b>Title:</b>'. $billhtml, $billtype);

      // retrieve the "All" page from Thomas and parse it
      $url = $info['source'] .'@@@L&summ2=m&';
      _uscongress_watchdog('screenscrape', $congress, $billtype . $billno, $url, t('GET Bill'), WATCHDOG_INFO);
      $result = drupal_http_request($url);
      if ($result->redirect_code == 200 || $result->code == 200) {
        $info['all'] = _uscongress_thomas_parse_all(str_replace("\n", " ", $result->data), $billtype);
      }
      else {
        _uscongress_watchdog('screenscrape', $congress, $billtype . $billno, $url, t('HTTP Error %code', array('%code' => $result->code)));
        $error = TRUE;
      }

      // retrieve the "Subject" page from Thomas and parse it
      $url = $info['source'] .'@@@J';
      $result = drupal_http_request($url);
      if ($result->redirect_code == 200 || $result->code == 200) {
        $info['subject'] = _uscongress_thomas_parse_subject(str_replace("\n", " ", $result->data));
      }
      else {
        _uscongress_watchdog('screenscrape', $congress, $billtype . $billno, $url, t('HTTP Error %code', array('%code' => $result->code)));
        $error = TRUE;
      }

      // save the bill
      if (_uscongress_thomas_add_bill_node($info) && !isset($error)) {
        _uscongress_set_import_status('screenscrape', $congress, $billtype . $billno, 1);
      }
      unset($error);

      // move onto the next bill, only retrieve this one again during updates
      variable_set('uscongress_last_'. $billtype, $info['number']);

      // stop if we've run out of time
      if (microtime(TRUE) - $start >= $time) {
        return -1;
      }
    }
    return 1;
  }
  return 0;
}

function _uscongress_thomas_parse_listing_summary($html, $billtype) {
  $details = explode('<br>', $html);
  foreach ($details as $detail) {
    if (preg_match(',<b>\s*(.*):\s*</b>\s*(.*)\s*,', $detail, $match)) {
      $info[strtolower($match[1])] = trim(strip_tags($match[2]));
    }
  }

  if (isset($info['sponsor'])) {
    $sponsor = $info['sponsor'];

    if ($info_sponsor = _uscongress_parse_sponsor($sponsor, $billtype)) {
      $info['sponsor'] = $info_sponsor;
    }
    else {
      $info['sponsor'] = array('title' => $sponsor);
    }

    $regex = '/introduced\s+([0-9\/]+)/i';
    if (preg_match($regex, $sponsor, $match)) {
      $info['introduced'] = $match[1];
    }

    $regex = '/cosponsors\s+\(([0-9]+)\)/i';
    if (preg_match($regex, $sponsor, $match)) {
      $info['cosponsor count'] = $match[1];
    }
  }

  if (isset($info['latest major action'])) {
    $action = explode('Status:', $info['latest major action']);
    $regex = '/^([0-9\/]+)\s+(.*)/';
    if (preg_match($regex, $action[0], $match)) {
      $info['latest major action'] = $match[2];
      $info['latest major action date'] = $match[1];
    }
    else {
      $info['latest major action'] = trim($action[0]);
    }
    if (isset($action[1])) {
      $info['status'] = trim($action[1]);
    }
  }
  return $info;
}

function _uscongress_parse_sponsor($sponsor, $billtype) {
  $regex = '/^((rep|sen)\s+)?([^,]+)(,\s*([^\[]+)(\s*\[([^\]]+)\])?)?/i';
  if (preg_match($regex, $sponsor, $match)) {
    $hall = isset($match[2]) ? (strtolower($match[2]) == 'rep' ? 'H' : 'S') : $billtype[0];
    $salutation = isset($match[2]) ? ucfirst($match[2]) : ($hall == 'S' ? 'Sen' : 'Rep');
    $lastname = trim($match[3]);
    $firstname = trim($match[5]);
    $info = array(
      'title' => "$salutation $lastname, $firstname",
      'lastname' => $lastname,
      'firstname' => $firstname,
      'hall' => $hall,
    );
    if ($hall == 'S') {
      $info['state'] = $match[7];
    }
    else {
      $info['district'] = $match[7];
    }
    return $info;
  }
}

function _uscongress_thomas_parse_all($html, $billtype) {
  // split the page into sections based on the query linking to the bill
  $details = explode('<a name=', $html);
  $header = array_shift($details);

  // parse the header
  $headersections = explode('<hr>', $header);
  if ($headersplit = preg_split(',<B>([^<]*)</B>,i', $headersections[3], -1, PREG_SPLIT_DELIM_CAPTURE)) {
    array_shift($headersplit);
    $headertitles = array();
    for ($i = 0; $i < count($headersplit); $i += 2) {
      $title = strtolower(preg_replace('/^s*|\s*:\s*$/', '', $headersplit[$i]));
      $headertitles[$title] = trim($headersplit[$i + 1]);
    }
    $info['title'] = strip_tags($headertitles['title']);
    $info['sponsor'] = _uscongress_parse_sponsor(strip_tags($headertitles['sponsor']), $billtype);
  }

  // parse the details
  foreach ($details as $detail) {
    if (preg_match('/^((\'([^\']+)\')|("([^"]+)"))(.*)/', $detail, $match)) {
      switch ($match[3] ? $match[3] : $match[5]) {
        case 'major actions':
          $info['major action'] = _uscongress_thomas_parse_majoractions($match[6]);
          break;
        case 'status':
          $info['action'] = _uscongress_thomas_parse_allactions($match[6]);
          break;
        case 'titles':
          $info['short_titles'] = _uscongress_thomas_parse_titles($match[6], 'short');
          break;
        case 'cosponsors':
          $info['cosponsor'] = _uscongress_thomas_parse_cosponsors($match[6], $billtype);
          break;
        case 'committees':
          $info['committee'] = _uscongress_thomas_parse_committees($match[6]);
          break;
        case 'rel-bill-detail':
          $info['related'] = _uscongress_thomas_parse_relatedbills($match[6]);
          break;
        case 'summary':
          $info['summary'] = _uscongress_thomase_parse_summary($match[6]);
          break;
        case 'amendments':
//        $info['ammendment'] = _uscongress_thomas_parse_ammendments($match[6]);
          break;
      }
    }
  }
  return $info;
}

function _uscongress_thomas_parse_subject($html) {
  if (preg_match(',CRS INDEX TERMS:\s*<ul>(.*)</ul>,', $html, $match)) {
    $rows = explode('<br>', $match[1]);
    array_shift($rows);
    foreach ($rows as $row) {
      $items[] = strip_tags($row);
    }
    return $items;
  }
}

function _uscongress_thomas_parse_majoractions($html) {
  $rows = explode('<TR>', $html);
  array_shift($rows);
  foreach ($rows as $row) {
    $cells = strip_tags($row);
    if (preg_match(',([0-9/]+)\s+(.*),', $cells, $match)) {
      $info[$match[1]] = trim($match[2]);
    }
  }
  return $info;
}

function _uscongress_thomas_parse_allactions($html) {
  $rows = explode('<dt>', $html);
  array_shift($rows);
  foreach ($rows as $row) {
    $cells = strip_tags($row);
    if (preg_match(',([0-9/]+(\s+[0-9]+:[0-9]+([ap]m)?)?)\s*:\s*(.*),', $cells, $match)) {
      $info[$match[1]] = trim($match[4]);
    }
  }
  return $info;
}

function _uscongress_thomas_parse_titles($html, $type) {
  $titles = explode('<li>', $html);
  $regex = '/^([^:]*'. $type .'[^:]*)\s*:\s*(.*)$/i';
  while ($fulltitle = array_pop($titles)) {
    if (preg_match($regex, $fulltitle, $matches)) {
      $type_titles = array();
      foreach (explode('<br>', $matches[2]) as $type_title) {
        $type_title = strip_tags($type_title);
        if ($type_title != '') {
          $type_titles[] = $type_title;
        }
      }
      if (count($type_titles)) {
        return $type_titles;
      }
    }
  }
}

function _uscongress_thomas_parse_cosponsors($html, $billtype) {
  $rows = explode('<br>', $html);
  array_shift($rows);
  foreach ($rows as $row) {
    if ($info_sponsor = _uscongress_parse_sponsor(strip_tags($row), $billtype)) {
      $info[] = $info_sponsor;
    }
  }
  return $info;
}

function _uscongress_thomas_parse_committees($html) {
  $rows = explode('<tr>', $html);
  array_shift($rows);
  array_shift($rows);
  foreach ($rows as $row) {
    $cells = explode('<td', $row);
    $action = trim(strip_tags('<td'. $cells[2]));
    $committee = trim(strip_tags('<td'. $cells[1]));
    $info[$committee] = $action;
  }
  return $info;
}

function _uscongress_thomas_parse_relatedbills($html) {
  $rows = explode('<tr>', $html);
  array_shift($rows);
  array_shift($rows);
  foreach ($rows as $row) {
    $cells = explode('<td', $row);
    $relationship = trim(strip_tags('<td'. $cells[2]));
    $billfullref = trim(strip_tags('<td'. $cells[1]));
    if (preg_match('/([^0-9]+)([0-9]+)/', $billfullref, $match)) {
      if ($billtype = _uscongress_get_billtype($match[1])) {
        $billtypes = _uscongress_thomas_billtypes();
        if (in_array($billtype, $billtypes)) {
          $billref = $billtype . $match[2];
          if (isset($info[$billref])) {
            $info[$billref] .= ', '. $relationship;
          }
          else {
            $info[$billref] = $relationship;
          }
        }
      }
    }
  }
  return $info;
}

function _uscongress_thomase_parse_summary($html) {
  if (preg_match_all(',<p>(.*)</p>,i', $html, $matches)) {
    $summary = array();
    foreach ($matches[0] as $para) {
      $summary[] = trim(strip_tags($para));
    }
    return implode("\n\n", $summary);
  }
}

function _uscongress_thomas_parse_ammendments($html) {
  // @TODO:
  $rows = explode('<P>', $html);
}

function _uscongress_thomas_new_node($type, $title) {
  return (object) array(
    'type' => $type,
    'title' =>  truncate_utf8($title, 128, TRUE, TRUE),
    'format' => 3,
    'status' => 1,
    'promote' => 0,
    'sticky' => 0,
    'revision' => 1,
    'name' => variable_get('uscongress_import_author', 'admin'),
    'log' => t('Imported by uscongress.module on '. date('g:i:s a')),
  );
}

function _uscongress_thomas_new_bill_node($info) {
  // create the node
  $billref = $info['billtype'] . $info['number'];
  if ($result = db_query(_uscongress_get_query_billref_sql(), $info['congressno'], $billref)) {
    if ($billnode = db_fetch_object($result)) {
      // don't get a cached version
      if ($node = node_load($billnode->nid, NULL, TRUE)) {
        $node->log = t('Updated by uscongress.module on '. date('g:i:s a'));

        // convert the node values to form values
        _content_widget_invoke('prepare form values', $node);
        $form_values = (array) $node;
      }
      // this is really a wierd error, and probably not worth checking for
      // but with all the importing and deleting in my testing, this happened once
      else {
        unset($node);
      }
    }
  }
  $title = isset($info['all']['title']) ? $info['all']['title'] : $info['listing']['title'];
  $short_title = count($info['all']['short_titles']) ? $info['all']['short_titles'][0] : $title;
  $short_title = _uscongress_get_fulltype($info['billtype']) . $info['number'] .': '. $short_title;
  if (empty($node)) {
    $node = _uscongress_thomas_new_node('usc_bill', $short_title);
    $form_values = (array) $node;
  }

  // set the path
  $form_values['path'] = 'bill/'. $info['congressno'] .'/'. $billref;

  // limit the titles to 128 characters
  $form_values['body'] = isset($info['all']['summary']) ? $info['all']['summary'] : $title;
  $form_values['title'] = substr($short_title, 0, 128);
  $form_values['og_description'] = substr($short_title, 0, 150);

  // always add this bill source
  $source = $info['source'];
  if (!isset($form_values['field_usc_sourceurl'])) {
    $form_values['field_usc_sourceurl'] = array();
  }
  foreach ($form_values['field_usc_sourceurl'] as $field_usc_sourceurl) {
    if ($field_usc_sourceurl['value'] == $source) {
      unset($source);
      break;
    }
  }
  if (isset($source)) {
    $form_values['field_usc_sourceurl'][] = array('value' => $source);
  }

  // add the basic node information
  $form_values['field_usc_billno'] = array(array('value' => $info['number']));
  $form_values['field_usc_billtype'] = array('key' => $info['billtype']);
  $form_values['field_usc_billref'] = array(array('value' => $billref));
  $form_values['field_usc_congressno'] = array('key' => $info['congressno']);

  // add the sponsor
  $sponsor = isset($info['all']['sponsor']['title']) ? $info['all']['sponsor'] : $info['listing']['sponsor'];
  $nid = db_result(db_query("SELECT nid FROM {node} WHERE type = 'usc_legislator' AND title = '%s'", $sponsor['title']));
  if (!$nid) {
    $nid = _uscongress_thomas_add_legislator_node($sponsor);
  }
  if ($nid) {
    $form_values['field_usc_sponsor'] = array('nids' => array($nid => $nid));
  }

  // add additional summary information
  if (isset($info['session'])) {
    $form_values['field_usc_session'] = array('key' => $info['session']);
  }
  if (isset($info['listing']['house reports'])) {
    $form_values['field_usc_housereport'] = array(array('value' => $info['listing']['house reports']));
  }
  if (isset($info['listing']['senate reports'])) {
    $form_values['field_usc_senatereport'] = array(array('value' => $info['listing']['senate reports']));
  }

  // add the committees
  if (count($info['all']['committee'])) {
    $field_usc_committee = array();
    foreach ($info['all']['committee'] as $committee => $action) {
      $nid = db_result(db_query("SELECT nid FROM {node} WHERE type='usc_committee' AND title='%s'", $committee));
      if (!$nid) {
        $nid = _uscongress_thomas_add_committee_node($committee);
      }
      $field_usc_committee[$nid] = $nid;
    }
    if (count($field_usc_committee)) {
      $form_values['field_usc_committee'] = array('nids' => $field_usc_committee);
    }
  }

  // add the cosponsors
  if (count($info['all']['cosponsor'])) {
    $field_usc_cosponsor = array();
    foreach ($info['all']['cosponsor'] as $cosponsor) {
      $nid = db_result(db_query("SELECT nid FROM {node} WHERE type = 'usc_legislator' AND title = '%s'", $cosponsor['title']));
      if (!$nid) {
        $nid = _uscongress_thomas_add_legislator_node($cosponsor);
      }
      $field_usc_cosponsor[$nid] = $nid;
    }
    if (count($field_usc_cosponsor)) {
      $form_values['field_usc_cosponsor'] = array('nids' => $field_usc_cosponsor);
    }
  }

  // add the action events
  if (count($info['all']['action'])) {
    foreach ($info['all']['action'] as $date => $action) {
      $nid = _uscongress_thomas_add_action($info['congressno'], $billref, $date, $action);
    }
  }
  if (count($info['all']['major action'])) {
    foreach ($info['all']['major action'] as $date => $action) {
      $nid = _uscongress_thomas_add_action($info['congressno'], $billref, $date, $action, TRUE);
      if ($nid && strtolower($action) == 'introduced in house') {
        $actionnode = node_load($nid);
        $form_values['field_usc_introduced'] = array(array('node_name' => $actionnode->title ." [nid: $nid]"));
      }
    }
  }

  // add the related bills
  if (count($info['all']['related'])) {
    $form_values['field_usc_related_billref'] = array();
    foreach ($info['all']['related'] as $billref => $billdescription) {
      $form_values['field_usc_related_billref'][] = array('value' => $billref);
    }
  }

  // add the ammendments

  return array($node, $form_values);
}

function _uscongress_drupal_execute($form_values, $node, $key) {
  // submit the form
  drupal_execute($node->type .'_node_form', $form_values, $node);

  // write form errors to the watchdog log
  static $num_prev_errors;
  $errors = form_get_errors();
  if (count($errors) != $num_prev_errors) {
    watchdog('error', t('Uscongress Import Error @type @key: @errors', array('@type' => $node->type, '@key' => $key, '@errors' => implode(',', array_slice($errors, $num_prev_errors)))));
    $num_prev_errors = count($errors);
  }
  else {
    // return the node id
    return isset($node->nid) ? $node->nid : _uscongress_last_insert_nid($node->type);
  }
}

function _uscongress_thomas_add_bill_node($info) {
  // insert/update the bill
  list($node, $form_values) = _uscongress_thomas_new_bill_node($info);
  $billref = $info['billtype'] . $info['number'];
  return _uscongress_drupal_execute($form_values, $node, $billref);
}

function _uscongress_thomas_add_legislator_node($person) {
  $node = _uscongress_thomas_new_node('usc_legislator', $person['title']);
  $form_values = (array) $node;
  foreach (array('state', 'hall') as $field) {
    if (isset($person[$field])) {
      $form_values['field_usc_'. $field] = array('key' => $person[$field]);
    }
  }
  foreach (array('district', 'lastname', 'firstname') as $field) {
    if (isset($person[$field])) {
      $form_values['field_usc_'. $field] = array(array('value' => $person[$field]));
    }
  }
  return _uscongress_drupal_execute($form_values, $node, $person['title']);
}

function _uscongress_thomas_add_committee_node($title) {
  $node = _uscongress_thomas_new_node('usc_committee', $title);
  return _uscongress_drupal_execute((array) $node, $node, $title);
}

function _uscongress_thomas_add_action($congressno, $billref, $date, $title, $major = 0) {
  // if there is an existing action, just return
  if ($result = db_query(_uscongress_thomas_get_query_billaction_sql(), $congressno, $billref, $date, $title)) {
    if ($node = db_fetch_object($result)) {
      return $node->nid;
    }
  }

  $node = _uscongress_thomas_new_node('usc_billaction', $title);
  $form_values = (array) $node;
  $form_values['field_usc_congressno'] = array('key' => $congressno);
  $form_values['field_usc_billref'] = array(array('value' => $billref));
  $form_values['field_usc_actiondate'] = array(array('value' => $date));
  $form_values['field_usc_major'] = array(array('value' => $major));
  return _uscongress_drupal_execute($form_values, $node, "$billref: $date");
}

function _uscongress_thomas_get_query_billaction_sql() {
  static $sql;
  if (!isset($sql)) {
    $field_usc_congressno = content_database_info(content_fields('field_usc_congressno', 'usc_billaction'));
    $field_usc_congressno_table = $field_usc_congressno['table'];
    $field_usc_congressno_column = $field_usc_congressno['columns']['value']['column'];
    $sql = 'SELECT n.* FROM {node} n LEFT JOIN {'. $field_usc_congressno_table .'} c ON c.vid = n.vid';

    $field_usc_billref = content_database_info(content_fields('field_usc_billref', 'usc_billaction'));
    $field_usc_billref_table = $field_usc_billref['table'];
    $field_usc_billref_column = $field_usc_billref['columns']['value']['column'];
    if ($field_usc_congressno_table != $field_usc_billref_table) {
      $sql .= ' LEFT JOIN {'. $field_usc_billref_table .'} b ON b.vid = n.vid';
    }

    $field_usc_actiondate = content_database_info(content_fields('field_usc_actiondate', 'usc_billaction'));
    $field_usc_actiondate_table = $field_usc_actiondate['table'];
    $field_usc_actiondate_column = $field_usc_actiondate['columns']['value']['column'];
    if ($field_usc_congressno_table != $field_usc_actiondate_table) {
      $sql .= ' LEFT JOIN {'. $field_usc_actiondate_table .'} d ON d.vid = n.vid';
    }

    $sql .= " WHERE $field_usc_congressno_column = %d AND $field_usc_billref_column = '%s' AND UNIX_TIMESTAMP($field_usc_actiondate_column) = UNIX_TIMESTAMP('%s') AND n.title = '%s'";
  }
  return $sql;
}
